@@ -1,11 +1,18 @@
 //!
 //! # CGlue
 //!
-//! [![Crates.io](https://img.shields.io/crates/v/cglue.svg)](https://crates.io/crates/cglue)
-//! [![API Docs](https://docs.rs/cglue/badge.svg)](https://docs.rs/cglue)
-//! [![Build and test](https://github.com/h33p/cglue/actions/workflows/build.yml/badge.svg)](https://github.com/h33p/cglue/actions/workflows/build.yml)
-//! [![MIT licensed](https://img.shields.io/badge/license-MIT-blue.svg)](https://github.com/h33p/cglue/blob/main/LICENSE)
-//! ![Rustc 1.45](https://img.shields.io/badge/rustc-1.45+-lightgray.svg)
+//! [![Crates.io]][crates] [![API Docs]][docs] [![Build and test]][workflows] [![MIT licensed]][license] [![Rustc 1.45]][rust]
+//!
+//! [Crates.io]: https://img.shields.io/crates/v/cglue.svg
+//! [crates]: https://crates.io/crates/cglue
+//! [API Docs]: https://docs.rs/cglue/badge.svg
+//! [docs]: https://docs.rs/cglue
+//! [Build and test]: https://github.com/h33p/cglue/actions/workflows/build.yml/badge.svg
+//! [workflows]: https://github.com/h33p/cglue/actions/workflows/build.yml
+//! [MIT licensed]: https://img.shields.io/badge/license-MIT-blue.svg
+//! [license]: https://github.com/h33p/cglue/blob/main/LICENSE
+//! [Rustc 1.45]: https://img.shields.io/badge/rustc-1.45+-lightgray.svg
+//! [rust]: https://blog.rust-lang.org/2020/07/16/Rust-1.45.0.html
 //!
 //! If all code is glued together, our glue is the safest on the market.
 //!
@@ -24,15 +31,17 @@
 //!   - [Plugin system](#plugin-system)
 //!   - [Working with cbindgen](#working-with-cbindgen)
 //!     - [Setup](#setup)
-//!     - [Automatic cleanup](#automatic-cleanup)
-//!     - [Cleanup C](#cleanup-c)
-//!     - [Cleanup C++](#cleanup-c-1)
+//!     - [cglue-bindgen](#cglue-bindgen)
 //! - [Limitations](#limitations)
+//!   - [Unstable feature](#unstable-feature)
+//! - [Projects using CGlue](#projects-using-cglue)
+//! - [Changelog](#changelog)
 //! <!-- /toc -->
 //!
 //! ## Overview
 //!
-//! CGlue offers an easy way to ABI (application binary interface) safety. Just a few annotations and your trait is ready to go!
+//! CGlue bridges Rust traits between C and other languages. It aims to be seamless to integrate -
+//! just add a few annotations around your traits, and they should be good to go!
 //!
 //! ```rust
 //! use cglue::*;
@@ -70,13 +79,11 @@
 //! }
 //! ```
 //!
-//! A CGlue object is ABI-safe, meaning it can be used across FFI-boundary - C code, or dynamically loaded Rust libraries. While Rust does not guarantee your code will work with 2 different compiler versions clashing, CGlue glues it all together in a way that works.
+//! Rust does not guarantee your code will work with
+//! neither [2 different compiler versions clashing](https://pastebin.com/raw/un1TbJCe), nor [any other minor changes](https://github.com/rust-lang/compiler-team/issues/457),
+//! CGlue glues it all together in a way that works.
 //!
-//! This is done by generating wrapper vtables (virtual function tables) for the specified trait, and creating an opaque object with matching table. Here is what's behind the `trait_obj` macro:
-//!
-//! ```ignore
-//! let obj = InfoPrinterBase::from(&mut info).into_opaque();
-//! ```
+//! This is done by generating wrapper vtables (virtual function tables) for the specified trait, and creating an opaque object with matching table.
 //!
 //! `cglue_trait` annotation generates a `InfoPrinterVtbl` structure, and all the code needed to construct it for a type implementing the `InfoPrinter` trait. Then, a `CGlueTraitObj` is constructed that wraps the input object and implements the `InfoPrinter` trait.
 //!
@@ -125,6 +132,8 @@
 //!
 //! // Implement the group for `Info` structure, defining
 //! // only that `InfoChanger` is optionally implemented.
+//! // This is not required if `unstable` feature is being used!
+//! # #[cfg(not(feature = "unstable"))]
 //! cglue_impl_group!(Info, InfoGroup, InfoChanger);
 //!
 //! # fn main() -> () {
@@ -155,10 +164,14 @@
 //!
 //! 3. Ability to wrap associated trait types into new CGlue trait objects and groups.
 //!
-//! 4. The above ability also works with mutable, and const reference associated type returns*.
+//! 4. The above ability also works with mutable, and const reference associated type returns[*](#associated-type-wrapping).
 //!
 //! 5. Generic traits and their groups.
 //!
+//! 6. [Library reference counting](#plugin-system).
+//!
+//! 7. Optional runtime ABI/API validation with [abi\_stable](https://crates.io/crates/abi_stable) (enable `layout_checks` feature).
+//!
 //! ## In-depth look
 //!
 //! ### Safety assumptions
@@ -167,9 +180,9 @@
 //! vtable functions will not be modified. It is being ensured through encapsulation of fields
 //! from anywhere by using hidden submodules. However, unverifiable users (C libraries) may still
 //! be able to modify the tables. This library assumes they are not malicious and does not
-//! perform any runtime verification. Currently there is no verification of API version mismatches,
-//! but it is in the plans to attempt to integrate with version checking systems available in
-//! [abi\_stable](https://crates.io/crates/abi_stable) crate.
+//! perform any runtime verification. API version mismatch checking with
+//! [abi\_stable](https://crates.io/crates/abi_stable) is an opt-in feature (requires rustc 1.46+,
+//! and abi_stable git, or 0.10.4 version).
 //!
 //! Other than 2 bits in [associated type wrapping](#associated-type-wrapping), this crate should
 //! be safe.
@@ -182,68 +195,82 @@
 //!
 //! `#[cglue_trait]` macro for `MyTrait` will generate the following important types:
 //!
-//! | Name | Purpose |
-//! --- | ---
-//! | `MyTraitBox` | Typedef for opaque owned CGlue object. Its container is a [`CBox<c_void>`](crate::boxed::CBox) |
-//! | `MyTraitCtxBox<D>` | Typedef for opaque owned CGlue object with an [opaque context](#plugin-system) `D`. Its container is a [`CtxBox<c_void, D>`](crate::boxed::CtxBox) |
-//! | `MyTraitNoCtxBox` | Typedef for opaque owned CGlue object. Its container is a [`CtxBox<c_void, NoContext>`](crate::boxed::CtxBox) |
-//! | `MyTraitArcBox` | Typedef for opaque owned CGlue object with an opaque reference counted context. Its container is a [`CtxBox<c_void, COptArc<c_void>>`](crate::boxed::CtxBox) |
-//! | `MyTraitMut` | Typedef for opaque by-mut-ref CGlue object. Its container is a `&mut c_void`. |
-//! | `MyTraitRef` | Typedef for opaque by-ref (const) CGlue object. Its container is a `&c_void`. |
-//! | `MyTraitAny<T, D>` | Typedef for opaque CGlue object. It can have any compatible container `T` dereferencing to `c_void`, with opaque context `D` |
+//! | Name | Purpose | Instance type | [Context](#plugin-system) |
+//! --- | --- | --- | ----
+//! | `MyTraitBox` | Regular owned CGlue object. | [`CBox<c_void>`](crate::boxed::CBox) | [`NoContext`](crate::trait_group::NoContext) |
+//! | `MyTraitCtxBox<Ctx>` | Owned CGlue object with a [context](#plugin-system). | [`CBox<c_void>`](crate::boxed::CBox) | `Ctx` |
+//! | `MyTraitArcBox` | Owned CGlue object with a reference counted context. | [`CBox<c_void>`](crate::boxed::CBox) | [`CArc<c_void>`](crate::arc::CArc) |
+//! | `MyTraitMut` | By-mut-ref CGlue object. | `&mut c_void`. | [`NoContext`](crate::trait_group::NoContext) |
+//! | `MyTraitCtxMut<Ctx>` | By-mut-ref CGlue object with a context. | `&mut c_void`. | `Ctx` |
+//! | `MyTraitArcMut` | By-mut-ref CGlue object with a reference counted context. | `&mut c_void`. | [`CArc<c_void>`](crate::arc::CArc) |
+//! | `MyTraitRef` | By-ref (const) CGlue object. | `&c_void`. | [`NoContext`](crate::trait_group::NoContext) |
+//! | `MyTraitCtxRef<Ctx>` | By-ref (const) CGlue object with a context. | `&c_void`. | `Ctx` |
+//! | `MyTraitArcRef` | By-ref (const) CGlue object with a reference counted context. | `&c_void`. | [`CArc<c_void>`](crate::arc::CArc) |
 //!
 //! Only opaque types provide functionality. Non-opaque types can be used as `Into` trait bounds
 //! and are required to type check trait bounds.
 //!
 //! These are the generic types needed for bounds checking:
 //!
-//! | Name | Purpose |
-//! --- | ---
-//! | `MyTraitBaseBox<F>` | Typedef for generic owned CGlue object. Its container is a [`CBox<F>`](crate::boxed::CBox) |
-//! | `MyTraitBaseCtxBox<F, C>` | Typedef for generic owned CGlue object with [some context](#plugin-system). Its container is a [`CtxBox<F, C>`](crate::boxed::CtxBox) |
-//! | `MyTraitBaseNoCtxBox<F>` | Typedef for generic owned CGlue object with some context. Its container is a [`CtxBox<F, NoContext>`](crate::boxed::CtxBox) |
-//! | `MyTraitBaseArcBox<F, C>` | Typedef for generic owned CGlue object with reference counted context. Its container is a [`CtxBox<F, COptArc<C>`](crate::boxed::CtxBox) |
-//! | `MyTraitBaseMut<F>` | Typedef for generic by-mut-ref CGlue object. Its container is a `&mut F`. |
-//! | `MyTraitBaseRef<F>` | Typedef for generic by-ref (const) CGlue object. Its container is a `&F`. |
-//! | `MyTraitBase<T, F, C, D>` | Base typedef for a CGlue object. It allows for any container type `T`, dereferencing to a concrete type `F`, with context `C` with its opaque version `D`. |
+//! | Name | Purpose | Instance type | Context |
+//! --- | --- | --- | ---
+//! | `MyTraitBaseBox<T>` | Base owned CGlue object. | [`CBox<T>`](crate::boxed::CBox) | [`NoContext`](crate::trait_group::NoContext) |
+//! | `MyTraitBaseCtxBox<T, Ctx>` | Base owned CGlue object with [some context](#plugin-system). | [`CBox<T>`](crate::boxed::CBox) | `Ctx` |
+//! | `MyTraitBaseArcBox<T, Ctx>` | Base owned CGlue object with reference counted context. | [`CBox<T>`](crate::boxed::CBox) | [`CArc<Ctx>`](crate::arc::CArc) |
+//! | `MyTraitBaseMut<T>` | Base by-mut-ref CGlue object. | `&mut T`. | [`NoContext`](crate::trait_group::NoContext) |
+//! | `MyTraitBaseRef<T>` | Typedef for generic by-ref (const) CGlue object. | `&T`. | [`NoContext`](crate::trait_group::NoContext) |
+//! | `MyTraitBase<Inst, Ctx>` | Base (non-opaque) CGlue object. It can have any compatible instance and context | `Inst` | `Ctx` |
 //!
 //! Finally, the following underlying types exist, but do not need to be interacted with in Rust:
 //!
 //! | Name | Purpose |
 //! --- | ---
-//! | `MyTraitVtbl<T, F, C, D>` | Table of all functions of the trait. Should be opaque to the user. |
-//! | `MyTraitOpaqueVtbl<D>` | Opaque version of the table. This is the type every object's table will have. |
-//! | `MyTraitRetTmp` | Structure for temporary return values. It should be opaque to the user. |
+//! | `MyTraitVtbl<C>` | Table of all functions of the trait. Should be opaque to the user. |
+//! | `MyTraitRetTmp<Ctx>` | Structure for temporary return values. It should be opaque to the user. |
+//!
+//! Instead, every opaque CGlue object implements `MyTraitOpaqueObj` trait, which contains the type
+//! of the vtable.
 //!
 //! `cglue_trait_group!` macro for `MyGroup` will generate the following main types:
 //!
-//! | Name | Purpose |
-//! --- | ---
-//! | `MyGroupBox` | Typedef for opaque owned CGlue trait group. Its container is a [`CBox<c_void>`](crate::boxed::CBox) |
-//! | `MyGroupCtxBox<D>` | Typedef for opaque owned CGlue trait group with [some context](#plugin-system) `D`. Its container is a [`CtxBox<c_void, D>`](crate::boxed::CtxBox) |
-//! | `MyGroupNoCtxBox` | Typedef for opaque owned CGlue trait group with no true context. Its container is a [`CtxBox<c_void, NoContext>`](crate::boxed::CtxBox) |
-//! | `MyGroupArcBox` | Typedef for opaque owned CGlue trait group with reference counted context. Its container is a [`CtxBox<c_void, COptArc<c_void>>`](crate::boxed::CtxBox) |
-//! | `MyGroupMut` | Typedef for opaque by-mut-ref CGlue trait group. Its container is a `&mut c_void`. |
-//! | `MyGroupRef` | Typedef for opaque by-ref (const) CGlue trait group. Its container is a `&c_void`. |
-//! | `MyGroupAny<T, D>` | Typedef for opaque CGlue trait group. It can have any container. |
+//! | Name | Purpose | Instance type | Context |
+//! --- | --- | --- | ---
+//! | `MyGroupBox` | Owned CGlue trait group. | [`CBox<c_void>`](crate::boxed::CBox) | [`NoContext`](crate::trait_group::NoContext) |
+//! | `MyGroupCtxBox<Ctx>` | Owned CGlue trait group with [some context](#plugin-system). | [`CBox<c_void>`](crate::boxed::CBox) | `Ctx` |
+//! | `MyGroupArcBox` | Typedef for opaque owned CGlue trait group with reference counted context. | [`CBox<c_void>`](crate::boxed::CBox) | [`CArc<c_void>`](crate::arc::CArc) |
+//! | `MyGroupMut` | Typedef for opaque by-mut-ref CGlue trait group. | `&mut c_void`. | [`NoContext`](crate::trait_group::NoContext) |
+//! | `MyGroupCtxMut<Ctx>` | Typedef for opaque by-mut-ref CGlue trait group with a custom context. | `&mut c_void`. | `Ctx` |
+//! | `MyGroupArcMut` | Typedef for opaque by-mut-ref CGlue trait group with a reference counted context. | `&mut c_void`. | [`CArc<c_void>`](crate::arc::CArc) |
+//! | `MyGroupRef` | Typedef for opaque by-ref (const) CGlue trait group. | `&c_void`. | [`NoContext`](crate::trait_group::NoContext) |
+//! | `MyGroupCtxRef<Ctx>` | Typedef for opaque by-ref (const) CGlue trait group with a custom context. | `&c_void`. | `Ctx` |
+//! | `MyGroupArcRef` | Typedef for opaque by-ref (const) CGlue trait group with a reference counted context. | `&c_void`. | [`CArc<c_void>`](crate::arc::CArc) |
 //!
 //! Base types are as follows:
 //!
+//! | Name | Purpose | Instance type | Context |
+//! --- | --- | --- | ---
+//! | `MyGroupBaseBox<T>` | Base owned CGlue trait group. Its container is a [`CBox<T>`](crate::boxed::CBox) |
+//! | `MyGroupBaseCtxBox<T, Ctx>` | Base owned CGlue trait group with [some context](#plugin-system). | [`CBox<T>`](crate::boxed::CBox) | `Ctx` |
+//! | `MyGroupBaseArcBox<T, Ctx>` | Base owned CGlue trait group with reference counted context. | [`CBox<T>`](crate::boxed::CBox) | [`CArc<Ctx>`](crate::arc::CArc) |
+//! | `MyGroupBaseMut<T>` | Base by-mut-ref CGlue trait group. | `&mut T`. | [`NoContext`](crate::trait_group::NoContext) |
+//! | `MyGroupBaseCtxMut<T, Ctx>` | Base by-mut-ref CGlue trait group with a context. | `&mut T`. | `Ctx` |
+//! | `MyGroupBaseArcMut<T, Ctx>` | Base by-mut-ref CGlue trait group with a reference counted context. | `&mut T`. | [`CArc<Ctx>`](crate::arc::CArc) |
+//! | `MyGroupBaseRef<T>` | Base by-ref (const) CGlue trait group. | `&T`. | [`NoContext`](crate::trait_group::NoContext) |
+//! | `MyGroupBaseCtxRef<T, Ctx>` | Base by-ref (const) CGlue trait group with a context. | `&T`. | `Ctx` |
+//! | `MyGroupBaseArcRef<T, Ctx>` | Base by-ref (const) CGlue trait group with a reference counted context. | `&T`. | [`CArc<Ctx>`](crate::arc::CArc) |
+//! | `MyGroup<Inst, Ctx>` | Base definiton of the group. It needs to be manually made opaque. | `Inst` | `Ctx` |
+//!
+//! Container type (opaque to Rust users) that is placed within the group:
+//!
 //! | Name | Purpose |
 //! --- | ---
-//! | `MyGroupBaseBox<F>` | Typedef for generic owned CGlue trait group. Its container is a [`CBox<F>`](crate::boxed::CBox) |
-//! | `MyGroupBaseCtxBox<F, C>` | Typedef for generic owned CGlue trait group with [some context](#plugin-system) `C`. Its container is a [`CtxBox<F, C>`](crate::boxed::CtxBox) |
-//! | `MyGroupBaseNoCtxBox<F>` | Typedef for generic owned CGlue trait group with no context. Its container is a [`CtxBox<F, NoContext>`](crate::boxed::CtxBox) |
-//! | `MyGroupBaseArcBox<F, C>` | Typedef for generic owned CGlue trait group with reference counted context. Its container is a [`CtxBox<F, COptArc<C>>`](crate::boxed::CtxBox) |
-//! | `MyGroupBaseMut<F>` | Typedef for generic by-mut-ref CGlue trait group. Its container is a `&mut F`. |
-//! | `MyGroupBaseRef<F>` | Typedef for generic by-ref (const) CGlue trait group. Its container is a `&F`. |
-//! | `MyGroup<T, F, C, D>` | Base definiton of the group. It is not opaque and not usable yet. |
+//! | `MyGroupContainer<Inst, Ctx>` | Stores temporary return storage. Vtables are built for this type.
 //!
 //! And finally, the filler trait that is required for an object to be grouppable:
 //!
 //! | Name | Purpose |
 //! --- | ---
-//! | MyGroupVtableFiller | Trait that allows an object to specify which optional traits are available, through the use of `enable_trait` functions. |
+//! | `MyGroupVtableFiller` | Trait that allows an object to specify which optional traits are available, through the use of `enable_trait` functions. |
 //!
 //! The macro generation will also generate structures for all combinations of optional traits
 //! being used. For more convenient by-macro usage, the names of optional traits inside are sorted
@@ -446,6 +473,9 @@
 //!
 //! #### Manually implementing groups
 //!
+//! NOTE: This is not supported if [`unstable`](#unstable-feature) feature is enabled. Instead, you
+//! have to do nothing!
+//!
 //! It is also possible to manually implement the groups by implementing `MyGroupVtableFiller`. Here is what
 //! the above 2 macro invocations expand to:
 //!
@@ -474,41 +504,36 @@
 //! # }
 //! # cglue_trait_group!(GenGroup<T: Eq>, Getter<T>, { TA });
 //! # use core::ops::Deref;
-//! # use cglue::trait_group::{ContextRef, Opaquable};
+//! # use cglue::trait_group::{Opaquable};
+//! # #[cfg(not(feature = "unstable"))]
 //! impl<
 //!         'cglue_a,
-//!         // Container type
-//!         CGlueT: ContextRef<Context = CGlueC> + Deref<Target = GA<T>>,
-//!         // Base context type, that opaques to `CGlueD`
-//!         CGlueC: 'static + Clone + Send + Sync + Opaquable<OpaqueTarget = CGlueD>,
-//!         // Opaque context type, that has been short-circuit to opaque to self
-//!         CGlueD: 'static + Clone + Send + Sync + Opaquable<OpaqueTarget = CGlueD>,
-//!         // This is the user-provided Eq bound
+//!         CGlueInst: ::core::ops::Deref<Target = GA<T>>,
+//!         CGlueCtx: cglue::trait_group::ContextBounds,
 //!         T: Eq,
-//!     > GenGroupVtableFiller<'cglue_a, CGlueT, CGlueC, CGlueD, T> for GA<T>
+//!     > GenGroupVtableFiller<'cglue_a, CGlueInst, CGlueCtx, T> for GA<T>
 //! where
-//!     // When we want to enable TA, we must mark that the vtable can be generated
-//!     &'cglue_a TAVtbl<'cglue_a, CGlueT, GA<T>, CGlueC, CGlueD>: 'cglue_a + Default,
+//!     Self: TA,
+//!     &'cglue_a TAVtbl<'cglue_a, GenGroupContainer<CGlueInst, CGlueCtx, T>>:
+//!         'cglue_a + Default,
+//!     T: cglue::trait_group::GenericTypeBounds,
 //! {
 //!     fn fill_table(
-//!         table: GenGroupVtables<'cglue_a, CGlueT, GA<T>, CGlueC, CGlueD, T>,
-//!     ) -> GenGroupVtables<'cglue_a, CGlueT, GA<T>, CGlueC, CGlueD, T> {
+//!         table: GenGroupVtables<'cglue_a, CGlueInst, CGlueCtx, T>,
+//!     ) -> GenGroupVtables<'cglue_a, CGlueInst, CGlueCtx, T> {
 //!         table.enable_ta()
 //!     }
 //! }
+//! # #[cfg(not(feature = "unstable"))]
 //! impl<
 //!         'cglue_a,
-//!         // Container type
-//!         CGlueT: ContextRef<Context = CGlueC> + Deref<Target = GA<u64>>,
-//!         // Base context type, that opaques to `CGlueD`
-//!         CGlueC: 'static + Clone + Send + Sync + Opaquable<OpaqueTarget = CGlueD>,
-//!         // Opaque context type, that has been short-circuit to opaque to self
-//!         CGlueD: 'static + Clone + Send + Sync + Opaquable<OpaqueTarget = CGlueD>,
-//!     > GenGroupVtableFiller<'cglue_a, CGlueT, CGlueC, CGlueD, u64> for GA<u64>
+//!         CGlueInst: ::core::ops::Deref<Target = GA<u64>>,
+//!         CGlueCtx: cglue::trait_group::ContextBounds,
+//!     > GenGroupVtableFiller<'cglue_a, CGlueInst, CGlueCtx, u64> for GA<u64>
 //! {
 //!     fn fill_table(
-//!         table: GenGroupVtables<'cglue_a, CGlueT, GA<u64>, CGlueC, CGlueD, u64>,
-//!     ) -> GenGroupVtables<'cglue_a, CGlueT, GA<u64>, CGlueC, CGlueD, u64> {
+//!         table: GenGroupVtables<'cglue_a, CGlueInst, CGlueCtx, u64>,
+//!     ) -> GenGroupVtables<'cglue_a, CGlueInst, CGlueCtx, u64> {
 //!         table
 //!     }
 //! }
@@ -594,7 +619,7 @@
 //!
 //! // Generated vtable entry:
 //!
-//! with_slice: extern "C" fn(&CGlueF, slice: CSlice<usize>),
+//! with_slice: extern "C" fn(&CGlueC, slice: CSlice<usize>),
 //! ```
 //!
 //! `Option` types that can not have [nullable pointer optimization](https://doc.rust-lang.org/nomicon/ffi.html#the-nullable-pointer-optimization) are wrapped into [COption](crate::option::COption):
@@ -604,7 +629,7 @@
 //!
 //! // Generated vtable entry:
 //!
-//! non_npo_option: extern "C" fn(&CGlueF, opt: Option<usize>),
+//! non_npo_option: extern "C" fn(&CGlueC, opt: Option<usize>),
 //! ```
 //!
 //! `Result` is automatically wrapped into [CResult](crate::result::CResult):
@@ -614,7 +639,7 @@
 //!
 //! // Generated vtable entry:
 //!
-//! with_cresult: extern "C" fn(&CGlueF) -> CResult<usize, usize>,
+//! with_cresult: extern "C" fn(&CGlueC) -> CResult<usize, usize>,
 //! ```
 //!
 //! `Result` with [IntError](crate::result::IntError) type can return an integer code with `Ok` value written to a variable:
@@ -625,7 +650,7 @@
 //!
 //! // Generated vtable entry:
 //!
-//! with_int_result: extern "C" fn(&CGlueF, ok_out: &mut MaybeUninit<usize>) -> i32,
+//! with_int_result: extern "C" fn(&CGlueC, ok_out: &mut MaybeUninit<usize>) -> i32,
 //! ```
 //!
 //! All wrapping and conversion is handled transparently behind the scenes, with user's control.
@@ -724,8 +749,8 @@
 //!
 //! CGlue currently does not provide an out-of-the box plugin system, but there are primitives in
 //! place for relatively safe trait usage using dynamically loaded libraries. The core primitive is
-//! the `CtxBox` type - this type allows to house a context, such as a `libloading::Library` Arc
-//! which will be automatically cloned into every owned object the trait creates.
+//! a cloneable context, such as a libloading::Library` Arc, which will keep the library opened
+//! until all of the CGlue objects are dropped.
 //!
 //! ```
 //! use cglue::prelude::v1::*;
@@ -740,7 +765,7 @@
 //! # fn main() -> () {
 //! let root = ();
 //! // This could be a `libloading::Library` arc.
-//! let ref_to_count = CArc::from(()).into_opt();
+//! let ref_to_count = CArc::from(());
 //! // Merely passing a tuple is enough.
 //! let obj = trait_obj!((root, ref_to_count) as PluginRoot);
 //! // ...
@@ -788,10 +813,8 @@
 //! # fn main() -> () {
 //! let root = ();
 //! // This could be a `libloading::Library` arc.
-//! let ref_to_count = CArc::from(()).into_opt();
-//! // Construct a `CtxBox` to be more explicit.
-//! let wrapped = CtxBox::from((root, ref_to_count));
-//! let obj = trait_obj!(wrapped as PluginRoot);
+//! let ref_to_count = CArc::from(());
+//! let obj = trait_obj!((root, ref_to_count) as PluginRoot);
 //! let printer = obj.get_printer();
 //! // It is safe to drop obj now:
 //! std::mem::drop(obj);
@@ -806,17 +829,14 @@
 //! `IntError` types, and mark the trait with `#[int_result]`, which would prevent this particular
 //! issue from happening.
 //!
-//! Another note, any by-ref objects do not have a context attached to them - their context is a
-//! `NoContext` type. If there is a way to construct an owned object out of them, the said object
-//! will not have the original context passed through. If reference counted context is used, this
-//! problem is easy to be type checked, because the returned type will be `MyTraitNoCtxBox`, rather
-//! than `MyTraitArcBox`.
-//!
 //! ### Working with cbindgen
 //!
 //! [cbindgen](https://github.com/eqrion/cbindgen) can be used to generate C and C++ bindings.
 //! There is some important setup needed.
 //!
+//! In addition, [`cglue-bindgen`](https://crates.io/crates/cglue-bindgen) provides additional
+//! helper method generation, making working with CGlue from C/C++ much more convenient.
+//!
 //! #### Setup
 //!
 //! Firstly, create a `cbindgen.toml`, and make sure both cglue, and any crates using cglue are
@@ -845,108 +865,26 @@
 //! language = "C"
 //! ```
 //!
-//! #### Automatic cleanup
+//! Export any shortened typedefs that are not used by any of the extern C functions:
 //!
-//! cbindgen will generate mostly clean output, however, there is one special case it does not
-//! handle well - empty typedefs.
+//! ```toml
+//! [export]
+//! include = ["FeaturesGroupArcBox", "PluginInnerRef", "PluginInnerMut"]
+//! ```
+//!
+//! #### cglue-bindgen
 //!
 //! [`cglue-bindgen`](https://crates.io/crates/cglue-bindgen) is a cbindgen wrapper that attempts
 //! to automatically clean up the headers. It also adds an ability to automatically invoke nightly
-//! rust with `+nightly` flag. The change is simple - just move all cbindgen arguments after `--`:
+//! rust with `+nightly` flag, and also generates vtable wrappers for simpler usage. The change is
+//! simple - just move all cbindgen arguments after `--`:
 //!
 //! ```sh
 //! cglue-bindgen +nightly -- --config cbindgen.toml --crate your_crate --output output_header.h
 //! ```
 //!
-//! If something does not work, below are the steps for manual cleanup in both C and C++ modes.
-//!
-//! #### Cleanup C
-//!
-//! Open the output header, and notice these typedefs:
-//!
-//! ```c
-//! /**
-//!  * Type definition for temporary return value wrapping storage.
-//!  *
-//!  * The trait does not use return wrapping, thus is a typedef to `PhantomData`.
-//!  *
-//!  * Note that `cbindgen` will generate wrong structures for this type. It is important
-//!  * to go inside the generated headers and fix it - all RetTmp structures without a
-//!  * body should be completely deleted, both as types, and as fields in the
-//!  * groups/objects. If C++11 templates are generated, it is important to define a
-//!  * custom type for CGlueTraitObj that does not have `ret_tmp` defined, and change all
-//!  * type aliases of this trait to use that particular structure.
-//!  */
-//! typedef struct CloneRetTmp CloneRetTmp;
-//! ```
-//!
-//! Remove all usage of these types. Any variables in the structures with these types should not be
-//! generated as they are intentionally zero-sized. Finally, remove the typedefs.
-//!
-//! Then, you might notice the following typedef:
-//!
-//! ```c
-//! /**
-//!  * Describes absence of a context.
-//!  *
-//!  * This context is used for regular `CBox` trait objects as well as by-ref or by-mut objects.
-//!  */
-//! typedef struct NoContext NoContext;
-//! ```
-//!
-//! Remove it, as well as any usages of it. It works the same way as zero-sized RetTmp variables.
-//!
-//! #### Cleanup C++
-//!
-//! Similar error is present in C++ headers, but due to templates, cleanup is slightly different.
-//!
-//! 1. Remove all references to the incomplete types in the generated group structures.
-//!
-//! 2. Change all incomplete `struct TraitRetTmp;` definitions to `typedef void TraitRetTmp;`
-//!
-//! 3. Define a specialized type for `CGlueTraitObj` without the final value:
-//!
-//! ```cpp
-//! template<typename T, typename V, typename S>
-//! struct CGlueTraitObj {
-//!     T instance;
-//!     const V *vtbl;
-//!     S ret_tmp;
-//! };
-//!
-//! // Make sure it goes after the original declaration.
-//!
-//! template<typename T, typename V>
-//! struct CGlueTraitObj<T, V, void> {
-//!     T instance;
-//!     const V *vtbl;
-//! };
-//! ```
-//!
-//! Similar specialization should be done for the `CtxBox` type:
-//!
-//! ```cpp
-//! template<typename T>
-//! struct CtxBox<T, void> {
-//!     CBox<T> inner;
-//! };
-//! ```
-//!
-//! Finally, there usually is a wrongly generated `MaybeUninit` typedef. Replace it from this:
-//!
-//! ```cpp
-//! template<typename T = void>
-//! struct MaybeUninit;
-//! ```
-//!
-//! To this:
-//!
-//! ```cpp
-//! template<typename T = void>
-//! using MaybeUninit = T;
-//! ```
-//!
-//! Other than that, everything should be good to go!
+//! This wrapper is probably the most fragile part of CGlue - if something does not work, please
+//! open up an issue report. In the future, we will aim to integrate CGlue directly with cbindgen.
 //!
 //! ## Limitations
 //!
@@ -958,12 +896,37 @@
 //! 3. Custom generic arguments for cglue traits are not yet supported, but this is to be improved
 //!    upon.
 //!
-//! 4. There is no runtime validation of ABI differences caused by API changes. It is planned
-//!    for a future release, perhaps integrating with
-//!    [abi\_stable](https://crates.io/crates/abi_stable).
-//!
-//! 5. There probably are some corner cases when it comes to path imports. If you find any, please
+//! 4. There probably are some corner cases when it comes to path imports. If you find any, please
 //!    file an issue report :)
+//!
+//! ### Unstable feature
+//!
+//! `cglue_impl_group` may force you into making conservative optional trait choices, because it is
+//! currently not possible to specialize these cases with stable Rust features. But this is not
+//! always desirable. You can solve this, by enabling `unstable` feature.
+//!
+//! This feature makes `cglue_impl_group` a no-op, and automatically enables the widest set of
+//! traits for the given object.
+//!
+//! To use it you need to either:
+//!
+//! - `nightly` Rust compiler.
+//!
+//! - Set `RUSTC_BOOTSTRAP=try_default` environment variable when building.
+//!
+//! Do note, however, that **Rust's stability guarantees get invalidated** by either of these 2
+//! options.
+//!
+//! ## Projects using CGlue
+//!
+//! * [memflow](https://github.com/memflow/memflow)
+//!
+//! If you want your project to be added to the list, please open an issue report :)
+//!
+//! ## Changelog
+//!
+//! It is available in [CHANGELOG.md](https://github.com/h33p/cglue/blob/main/CHANGELOG.md) file.
+//!
 
 #![cfg_attr(not(feature = "std"), no_std)]
 extern crate no_std_compat as std;
@@ -983,20 +946,27 @@ pub mod trait_group;
 pub use ::cglue_macro::{
     as_mut, as_ref, cast, cglue_forward, cglue_forward_ext, cglue_impl_group, cglue_trait,
     cglue_trait_ext, cglue_trait_group, group_obj, int_result, into, no_int_result, return_wrap,
-    skip_func, trait_obj, wrap_with, wrap_with_group, wrap_with_group_mut, wrap_with_group_ref,
-    wrap_with_obj, wrap_with_obj_mut, wrap_with_obj_ref,
+    skip_func, trait_obj, vtbl_only, wrap_with, wrap_with_group, wrap_with_group_mut,
+    wrap_with_group_ref, wrap_with_obj, wrap_with_obj_mut, wrap_with_obj_ref,
 };
 
+#[cfg(feature = "unstable")]
+pub use try_default::TryDefault;
+
 pub mod ext {
-    // Built-in external traits.
+    //! # Built-in external traits.
+    //!
+    //! All of the traits implemented here are usable in trait groups and objects.
     cglue_macro::cglue_builtin_ext_traits!();
 }
 
 pub mod prelude {
+    //! # Import most commonly used types.
+
     pub mod v1 {
         pub use crate::{
-            arc::{CArc, COptArc},
-            boxed::{CBox, CtxBox},
+            arc::CArc,
+            boxed::CBox,
             callback::{Callback, FeedCallback, FromExtend, OpaqueCallback},
             forward::{Forward, ForwardMut, Fwd},
             iter::CIterator,
@@ -1007,6 +977,12 @@ pub mod prelude {
             trait_group::Opaquable,
             *,
         };
+
+        #[cfg(feature = "unstable")]
+        pub use try_default::TryDefault;
+
+        #[cfg(feature = "layout_checks")]
+        pub use crate::trait_group::VerifyLayout;
     }
 }
 
